package ru.dbotthepony.mc.otm.capability.android;

import net.minecraft.client.Minecraft;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Player;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.network.android.AndroidResearchRequestPacket;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class AndroidFeatureResearchNode {
	public final AndroidFeatureType<?> type;
	public final AndroidFeatureResearchCost cost;

	public final Set<AndroidFeatureType<?>> predecessors = new HashSet<>();

	protected final Component name;

	public AndroidFeatureResearchNode(AndroidFeatureType<?> type, AndroidFeatureResearchCost cost) {
		this.type = type;
		this.cost = cost;
		name = new TranslatableComponent("android_feature." + type.getRegistryName().getNamespace() + "." + type.getRegistryName().getPath());
	}

	public AndroidFeatureResearchNode addPredecessor(AndroidFeatureType<?> value) {
		predecessors.add(value);
		return this;
	}

	public boolean canResearch(Player ply, AndroidCapabilityPlayer cap) {
		for (var prec : predecessors) {
			if (!cap.hasFeature(prec))
				return false;
		}

		return cost.matches(ply);
	}

	public void research(Player ply, AndroidCapabilityPlayer cap) {
		if (cap.hasFeature(type))
			return;

		if (!canResearch(ply, cap))
			return;

		cost.research(ply);
		cap.addFeature(type);
	}

	public void researchClient() {
		var ply = Minecraft.getInstance().player;

		ply.getCapability(MatteryCapability.ANDROID).ifPresent(cap -> {
			if (cap.hasFeature(type))
				return;

			if (!canResearch(ply, (AndroidCapabilityPlayer) cap))
				return;

			MatteryNetworking.CHANNEL.sendToServer(new AndroidResearchRequestPacket(type));
		});
	}

	public Component getName() {
		return name;
	}

	public List<Component> getTooltip() {
		return List.of(getName());
	}
}
