package ru.dbotthepony.mc.otm.network.android;

import net.minecraft.client.Minecraft;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.server.level.ServerPlayer;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.fml.DistExecutor;
import net.minecraftforge.fmllegacy.network.NetworkEvent;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.android.AndroidFeature;
import ru.dbotthepony.mc.otm.capability.android.AndroidFeatureType;
import ru.dbotthepony.mc.otm.capability.android.IAndroidCapability;

import java.util.Objects;
import java.util.function.Supplier;

public record AndroidResearchRequestPacket(AndroidFeatureType<?> feature) {
	public AndroidResearchRequestPacket(AndroidFeatureType<?> feature) {
		this.feature = feature;
	}

	public AndroidResearchRequestPacket(AndroidFeature feature) {
		this(feature.type);
	}

	public void write(FriendlyByteBuf buffer) {
		buffer.writeInt(Registry.ANDROID_FEATURES().getID(feature));
	}

	public void play(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);
		context.get().enqueueWork(() -> context.get().getSender().getCapability(MatteryCapability.ANDROID).ifPresent(cap -> this.playServer((AndroidCapabilityPlayer) cap, (ServerPlayer) context.get().getSender())));
	}

	public void playServer(AndroidCapabilityPlayer cap, ServerPlayer ply) {
		if (cap.hasFeature(feature))
			return;

		var node = feature.getResearchNode();

		if (node == null)
			return;

		node.research(ply, cap);
	}

	public static AndroidResearchRequestPacket read(FriendlyByteBuf buffer) {
		return new AndroidResearchRequestPacket(Objects.requireNonNull(Registry.ANDROID_FEATURES().getValue(buffer.readInt())));
	}
}
